:: =============================================================================
:: =                Copyright (C) 2015-2017 ProDigital Software.               =
:: =                            All Rights Reserved.                           =
:: =                                                                           =
:: =  This computer program is protected by copyright law and international    =
:: =  treaties.  Unauthorized reproduction or redistribution of this program,  =
:: =  or any portion of it, may result in severe civil or criminal penalties,  =
:: =  and will be prosecuted accordingly.                                      =
:: =============================================================================
::
::+=============================================================================
::
:: Name:        HostsCompiler
:: Ident:       3.18
:: Type:        User Command Task
:: Language:    MS-DOS Batch
:: Index:       Gather lists of badware servers and compile into a hosts file
::
::
:: Function:
::
::      This script gathers several well-managed lists of badware sites found
::      online and reduces the list into hosts file that will help protect 
::      your local machine.  
::
::		With such a list in place your browsing experience will be faster, as
::		ads and trackers will simply not be loaded, and you'll be protected from
::		inadvertently visiting known sites that serve malware.
::
::		It is possible this will cause some legitimate sites that you visit not 
::		to provide the information you're used to seeing.  Forbes, for example, has
::		added scripting to their web pages that put up messages asking you to disable
::		your ad blocker.  My advice:  If you run into such a site after generating
::		a hosts file with this tool, just browse elsewhere.  There's plenty of
::		other information out there!
::
::
:: Synopsis:
::
::      HostsCompiler  NoPrompt  NoHostsUpdate
::
::      NoPrompt -      (Optional) If specified, will avert the request for confirmation.  
::                      This is useful for unattended scheduled runs.
::
::      NoHostsUpdate - (Optional) If specified, will leave a reference copy of the
::                      generated hosts file in C:\ProgramData\HostsCompiler but will 
::                      not update the actual system hosts file.
::
::
::		Suggest scheduling this job with a command line similar to:
::
::		HostsCompiler NoPrompt >>C:\Some\Path\HostsCompiler.log 2>&1
::
::
:: Additional Information:
::
::		This script is distributed with several additional tools that are needed for
::		its operation.  Unzip the whole set of tools provided together into one folder 
::		and it should work for you.
::
::		Malware domain source information includes:
::
::		http://winhelp2002.mvps.org/hosts.txt
::		http://www.malwaredomainlist.com/hostslist/hosts.txt
::		https://adaway.org/hosts.txt
::		http://someonewhocares.org/hosts/
::		http://pgl.yoyo.org/adservers/
::		http://www.quero.at/adblock-hosts-edge.php
::		http://sysctl.org/cameleon/hosts.win
::
::		The information from above is reduced for uniqueness and appended to your
::		original hosts file.  This can be done multiple times, as the script removes
::		prior information it has automatically added to hosts.  Don't add anything
::		after the block added by this script or it will be lost!
::
::
::		If you have a deny-outgoing-connections-by-default firewall setup, you will
::		need to allow the wget.exe program to access the Internet via TCP port 80.
::
::-=============================================================================
::
::  Start of process.
::
    @ECHO OFF
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Ensure HOSTS file exists
::
    IF EXIST "%SystemRoot%\System32\drivers\etc\hosts." GOTO CHECK_TOOLS
::
    ECHO.
    ECHO ERROR:  File "%SystemRoot%\System32\drivers\etc\hosts." cannot be located.  Unable to continue.
    GOTO CLEAN_UP
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Ensure all the associated tools are available via the path or in the folder with the script
::
:CHECK_TOOLS
::
    SET _HC_ORIGINAL_PATH=%Path%
    SET Path=%Path%;%~dp0
::
    which tool.cannot.be.found.on.this.system >nul 2>&1
    IF ERRORLEVEL 2 ECHO No which in (.;%PATH%)
    IF ERRORLEVEL 2 GOTO TOOLS_ERROR
::
    FOR %%T IN (diff,grep,gsort,sed,showtime,sleep,unzip,wget) DO (
        which %%T 2>&1 1>nul
        IF ERRORLEVEL 1 GOTO TOOLS_ERROR
    )
::
    GOTO GOT_TOOLS
::
:TOOLS_ERROR
::
    ECHO.
    ECHO ERROR:  Not all the tools needed could be found either in the path or in the folder with the script.  Unable to continue.
    GOTO CLEAN_UP
::
:GOT_TOOLS
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Get a unique temporary area subfolder to work with
::
    SET _HC_ORIGINAL_TEMP=%TEMP%
::
    ShowTime "SET TEMP=%_HC_ORIGINAL_TEMP%\%%H%%M%%S.%%%%03d" >"%_HC_ORIGINAL_TEMP%\_HC_Doit.bat"
    CALL "%_HC_ORIGINAL_TEMP%\_HC_Doit.bat"
    DEL "%_HC_ORIGINAL_TEMP%\_HC_Doit.bat" >nul 2>&1
::
::  Work in the new TEMP area so we can drop temporary files as needed.
::
    MD "%TEMP%" >nul 2>&1
    PUSHD "%TEMP%"
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Check for incorrect parameters.
::
    IF NOT "%3" == "" GOTO DISPLAY_HELP
::
    IF /I "%1" == ""                IF /I "%2" == ""                GOTO INTERPRET_PARAMETERS
    IF /I "%1" == "NOPROMPT"        IF /I "%2" == ""                GOTO INTERPRET_PARAMETERS
    IF /I "%1" == "NOHOSTSUPDATE"   IF /I "%2" == ""                GOTO INTERPRET_PARAMETERS
    IF /I "%1" == "NOPROMPT"        IF /I "%2" == "NOHOSTSUPDATE"   GOTO INTERPRET_PARAMETERS
    IF /I "%1" == "NOHOSTSUPDATE"   IF /I "%2" == "NOPROMPT"        GOTO INTERPRET_PARAMETERS
::
:DISPLAY_HELP
::
    ECHO.
    ECHO ERROR:  Incorrect parameters specified.
    ECHO.
    ECHO Function:
    ECHO.
    ECHO     This script gathers several well-managed lists of badware sites from 
    ECHO     online sources and reduces the list into a hosts file that will help 
    ECHO     protect your local machine from visiting known malware sites.
    ECHO.
    ECHO.
    ECHO     HostsCompiler  NoPrompt  NoHostsUpdate
    ECHO.
    ECHO     NoPrompt -     (Optional) If specified, will avert the request for confirmation.  
    ECHO                    This is useful for unattended scheduled runs.
    ECHO.
    ECHO    NoHostsUpdate - (Optional) If specified, will leave a reference copy of the
    ECHO                    generated hosts file in C:\ProgramData\HostsCompiler but will 
    ECHO                    not update the actual system hosts file.
    ECHO.
::
    GOTO CLEAN_UP
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:INTERPRET_PARAMETERS
::
    SET _HC_NOPROMPT_OPTION=
    IF /I "%1" == "NOPROMPT" SET _HC_NOPROMPT_OPTION=TRUE
    IF /I "%2" == "NOPROMPT" SET _HC_NOPROMPT_OPTION=TRUE
::
    SET _HC_NOHOSTSUPDATE_OPTION=
    IF /I "%1" == "NOHOSTSUPDATE" SET _HC_NOHOSTSUPDATE_OPTION=TRUE
    IF /I "%2" == "NOHOSTSUPDATE" SET _HC_NOHOSTSUPDATE_OPTION=TRUE
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:CONFIRM_RUN
::
    IF DEFINED _HC_NOPROMPT_OPTION GOTO START_RUN
::
    SET /P _HC_CONTINUE="This script will acquire input data from sources online and compile a hosts file.  Continue (Y/N)? "
    IF /I NOT "%_HC_CONTINUE%" == "Y" GOTO ABORT_RUN
::
    SET _HC_INTERACTIVE=YES
    GOTO START_RUN
::
:ABORT_RUN
    ECHO Aborted run.
    ECHO.
    GOTO CLEAN_UP
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:START_RUN
::
::  Ensure we have a working area for list acquisition.
::
    MD "%PROGRAMDATA%\HostsCompiler" >nul 2>&1
::
    ECHO.
    ECHO Retrieving malware domain information from various sources...
    ECHO.
::
::  Note that this banner is used by the logic that trims out the prior hosts data, so think twice before changing it!
::
    SET "_HC_BANNER=# Added by HostsCompiler.bat on "
::
    ShowTime "%_HC_BANNER%%%s from:">Banner.txt
    ECHO.>>Banner.txt
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  All known HOSTS files are either available directly in TXT format, or are compressed text files in ZIP format.
::
::  If the HOSTS file is available directly in text format, add the line for that source below in the format:
::  		CALL :_GET_HOST_LIST List_Name "List_URL"
::
::  The List_Name should not contain any spaces, and will be used to create reference copies of the data 
::   from the blacklisted sites from the current and last version of the HOST file obtained from that site, 
::   such as %PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list.
::  
::  If the HOSTS file is only available in ZIP format, add a third parameter to the Function Call, which is 
::   the name of the file inside the ZIP:
::
::  		CALL :_GET_HOST_LIST List_Name "List_URL" File_Name_Inside_Zip
::
::  Add as many valid sources here as you like by adding lines in the same format.
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
    CALL :_GET_HOST_LIST mvps				"http://winhelp2002.mvps.org/hosts.txt"
    CALL :_GET_HOST_LIST malwaredomain		"http://www.malwaredomainlist.com/hostslist/hosts.txt"
    CALL :_GET_HOST_LIST adaway				"https://adaway.org/hosts.txt"
    CALL :_GET_HOST_LIST someonewhocares	"http://someonewhocares.org/hosts/hosts"
    CALL :_GET_HOST_LIST peterlowe			"http://pgl.yoyo.org/as/serverlist.php?hostformat=hosts&showintro=0&mimetype=plaintext"
    CALL :_GET_HOST_LIST quero				"http://www.quero.at/download/adblock-hosts.zip" hosts.
    CALL :_GET_HOST_LIST cameleon			"http://sysctl.org/cameleon/hosts.win"
::
::  WARNING!
::
::  The following sites can supply even more blacklist entries - hundreds of thousands, actually.
::   Unfortunately, this results in a hosts file so huge that it causes system problems, such as
::   virtually endless looping of the dnscache service.
::
::  Only if you have a pure test system and are doing some kind of robustness testing should you 
::   consider commenting these in!  You have been warned!
::
::	CALL :_GET_HOST_LIST hphosts "http://hosts-file.net/download/hosts.zip" hosts.txt
::	CALL :_GET_HOST_LIST hphosts-partial "http://hosts-file.net/hphosts-partial.txt"
::
::  Add new sources here by adding lines like the above
::
    ECHO.>>Banner.txt
::
    ECHO.>EndBanner.txt
    ECHO # End of entries Added by HostsCompiler.bat>>EndBanner.txt
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  All the files have been retrieved and formatted into just server names
::
::  Get all just-generated lists into one file
::
    COPY /B *.list NewHostsData.tmp >nul
::
::  Sort and eliminate duplicates
::
    gsort -f -u NewHostsData.tmp >NewHostsData.srt
    IF ERRORLEVEL 1 GOTO EDIT_FAIL
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Save a copy of the prior hosts file for reference, removing "read-only" and "hidden" attributes if necessary.
::
    IF EXIST "%PROGRAMDATA%\HostsCompiler\hosts.last" SET _HC_DO_DIFF=YES
::
::  Get the original part of the hosts file sans anything previously added by this script.
::  If we can't find anything previously added by this script, use the entire hosts file.
::
    sed -r -n -e "/^%_HC_BANNER%/q;p" "%PROGRAMDATA%\HostsCompiler\hosts." >BaseHostsFile.txt
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Note that we remove comment characters off the front of entries because if they're commented out then chances are
::  they've been whitelisted by the user and we want to exclude them from the added dataset.
::
    ECHO # The following are whitelisted entries found commented-out in the previous hosts file.>WhiteListBanner.txt
    ECHO # These are excluded from the generated blacklist.>>WhiteListBanner.txt
    ECHO.>>WhiteListBanner.txt
    grep -E "^#[ \t]*[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[ \t]+[-a-zA-Z0-9_]+\.[-a-zA-Z0-9_]" "%PROGRAMDATA%\HostsCompiler\hosts." | gsort -f -u >WhiteList.txt
::
::  Isolate just the server names by removing comments, IPv6, blank lines, and any IP address prefixes.
::
    sed -r -e "s/#.*$//g" -e "s/[ \t]+/ /g" -e "/^::/d" -e "/^ *$/d" -e "s/ +$//g" -e "s/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ //g" BaseHostsFile.txt >Exclusions.tmp
    sed -r -e "s/^#[ \t]*([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+[ \t]+[-a-zA-Z0-9_]+\.[-a-zA-Z0-9_])/\1/g" -e "s/#.*$//g" -e "s/[ \t]+/ /g" -e "s/ +$//g" -e "/^::/d" -e "/^ *$/d" -e "s/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ //g" WhiteList.txt >>Exclusions.tmp
::
::  Create a sorted exclusion list that can be compared to the new entries being added.
::
    gsort -f -u Exclusions.tmp >Exclusions.srt
::
::  Make sure we're not adding any entries that are already in the BaseHostsFile.
::
    diff -i -u Exclusions.srt NewHostsData.srt | grep -E "^\+" | sed -e "1d" -r -e "s/^.//" >NewHostsData.list
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  At this point we have a clean set of unique, sorted server names.  Set hosts file formatting to generate the blacklist.
::
    ECHO.>BlackListBanner.txt
    ECHO # The generated blacklist follows.   Enjoy your enhanced security!>>BlackListBanner.txt
    ECHO.>>BlackListBanner.txt
    ECHO #  Note:  If you want to exclude any entry from the blacklist, comment it out by placing>>BlackListBanner.txt
    ECHO #         a single # character in front of it.  The name resolution for that site will>>BlackListBanner.txt
    ECHO #         then be allowed and it will be excluded from future HostCompiler runs.>>BlackListBanner.txt
    ECHO.>>BlackListBanner.txt
::
    sed -r -e "s/^/0.0.0.0 /" NewHostsData.list >BlackList.txt
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Create the final hosts file - /B switch averts a hex 1A from being tacked on the end
::  Try several times, as it has been seen that the hosts file may be being accessed by the system.
::
    COPY /B BaseHostsFile.txt+Banner.txt+WhiteListBanner.txt+WhiteList.txt+BlackListBanner.txt+BlackList.txt+EndBanner.txt "%PROGRAMDATA%\HostsCompiler\hosts." >nul
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Success!
::
    ECHO.
    ECHO hosts vs. hosts.last
    ECHO -----------------------------------------------------------------------------------------------------------------------------
    if "%_HC_DO_DIFF%" == "YES" diff -y --suppress-common-lines --width=125 "%PROGRAMDATA%\HostsCompiler\hosts." "%PROGRAMDATA%\HostsCompiler\hosts.last"
    if NOT "%_HC_DO_DIFF%" == "YES" grep -c $ BlackList.txt | sed -r -e "s/$/ new entries added to your hosts file.  Enjoy your improved browsing and security!/"
    ECHO -----------------------------------------------------------------------------------------------------------------------------
::
::  Save a copy of the file just generated so it can be checked against the latest file next time this is run.
::
    COPY "%PROGRAMDATA%\HostsCompiler\hosts." "%PROGRAMDATA%\HostsCompiler\hosts.last" >nul
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Determine whether to update the actual system hosts. file.  Avoid doing it if the optional DontUpdateHosts command line parameter is specified.
::   This leaves a reference copy in the C:\ProgramData\HostsCompiler folder.
::
    IF NOT DEFINED _HC_NOHOSTSUPDATE_OPTION GOTO UPDATE_ACTUAL_HOSTS_FILE
::
    ECHO.
    ECHO NoHostsUpdate parameter specified:  A reference copy of new hosts file has been left in:  %PROGRAMDATA%\HostsCompiler\hosts.
    ECHO.
::
    GOTO LOG_FINAL
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  Determine current attributes so we can change them back after accessing hosts.
::
:UPDATE_ACTUAL_HOSTS_FILE
::
    COPY %SystemRoot%\System32\drivers\etc\hosts. "%PROGRAMDATA%\HostsCompiler\hosts.last" >nul
    IF ERRORLEVEL 1 GOTO EDIT_FAIL
::
    SET _HC_HOSTS_ATTRIB_RESTORE=
    SET _HC_HOSTS_ATTRIB_TEMP=
::
    FOR %%? IN (%SystemRoot%\System32\drivers\etc\hosts.) DO SET _HC_HOSTS_ATTRIB=%%~a?
::
    SET _HC_DUMMY=%_HC_HOSTS_ATTRIB:h=%
    IF NOT "%_HC_DUMMY%" == "%_HC_HOSTS_ATTRIB%" (
        SET _HC_HOSTS_ATTRIB_TEMP=%_HC_HOSTS_ATTRIB_TEMP% -h
        SET _HC_HOSTS_ATTRIB_RESTORE=%_HC_HOSTS_ATTRIB_RESTORE% +h
    )
::
    SET _HC_DUMMY=%_HC_HOSTS_ATTRIB:r=%
    IF NOT "%_HC_DUMMY%" == "%_HC_HOSTS_ATTRIB%" (
        SET _HC_HOSTS_ATTRIB_TEMP=%_HC_HOSTS_ATTRIB_TEMP% -r
        SET _HC_HOSTS_ATTRIB_RESTORE=%_HC_HOSTS_ATTRIB_RESTORE% +r
    )
::
    IF NOT "%_HC_HOSTS_ATTRIB_TEMP%" == "" ATTRIB %_HC_HOSTS_ATTRIB_TEMP% %SystemRoot%\System32\drivers\etc\hosts.
::
::  Now attempt to overwrite the system hosts file with some retries in case it's blocked from use (seen often in practice)
::
    COPY /B "%PROGRAMDATA%\HostsCompiler\hosts." %SystemRoot%\System32\drivers\etc\hosts. >nul
    IF NOT ERRORLEVEL 1 GOTO NEW_HOSTS_CREATED
::
    sleep 15
    COPY /B "%PROGRAMDATA%\HostsCompiler\hosts." %SystemRoot%\System32\drivers\etc\hosts. >nul
    IF NOT ERRORLEVEL 1 GOTO NEW_HOSTS_CREATED
::
    sleep 30
    COPY /B "%PROGRAMDATA%\HostsCompiler\hosts." %SystemRoot%\System32\drivers\etc\hosts. >nul
    IF NOT ERRORLEVEL 1 GOTO NEW_HOSTS_CREATED
::
    sleep 60
    COPY /B "%PROGRAMDATA%\HostsCompiler\hosts." %SystemRoot%\System32\drivers\etc\hosts. >nul
    IF NOT ERRORLEVEL 1 GOTO NEW_HOSTS_CREATED
::
    sleep 60
    COPY /B "%PROGRAMDATA%\HostsCompiler\hosts." %SystemRoot%\System32\drivers\etc\hosts. >nul
    IF NOT ERRORLEVEL 1 GOTO NEW_HOSTS_CREATED
::
::  Failed
::
    GOTO RESTORE_HOSTS
::
::  Succeeded
::
:NEW_HOSTS_CREATED
::
::  Restore original attributes of HOSTS if necessary, reset PATH, and delete temp files.
::
    IF EXIST %SystemRoot%\System32\drivers\etc\hosts. IF NOT "%_HC_HOSTS_ATTRIB_RESTORE%" == "" ATTRIB %_HC_HOSTS_ATTRIB_RESTORE% %SystemRoot%\System32\drivers\etc\hosts.
::
::  Now cause a name resolution that won't be in the cache so that the dnscache service will load up the file just created.
::
    PING -n 1 xxx.yyyyyy.zzz >nul 2>&1
::
    ECHO.
    DIR %SystemRoot%\System32\drivers\etc\hosts.*
::
    GOTO LOG_FINAL
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
::  If an error occurred in creating the actual hosts file, restore the file we started with
::
:RESTORE_HOSTS
::
    COPY "%PROGRAMDATA%\HostsCompiler\hosts.last" "%PROGRAMDATA%\HostsCompiler\hosts."
::
:EDIT_FAIL
::
    ECHO.
    ECHO ERROR:  Could not process one or more of the data files.  This is as far as we got:
    DIR
    ECHO.
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:LOG_FINAL
::
    ECHO.
    ShowTime HostsCompiler.bat finished on %%s.
    ECHO.
    ECHO ====================================================================================================================================
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:CLEAN_UP
::
    IF NOT "%_HC_ORIGINAL_PATH%" == "" SET Path=%_HC_ORIGINAL_PATH%
::
    IF "%_HC_ORIGINAL_TEMP%" == "" GOTO TEMP_CLEANUP_DONE
    POPD
    RD /S /Q %TEMP% >nul
    SET TEMP=%_HC_ORIGINAL_TEMP%
::
:TEMP_CLEANUP_DONE
::
    IF "%_HC_INTERACTIVE%" == "YES" PAUSE
::
::  Delete environment variables, all of which start with "_HC_"
::
    FOR /F "tokens=1* delims==" %%G IN ('"SET "_HC_" 2>nul"') DO (SET "%%G=")
    GOTO END_OF_PROCEDURE
::
::==============================================================================
::
::  Callable function to retrieve hosts data from a site online and format it.
::
::  Keep the last received lists from each of the managed list sites.  
::  If the list can't be downloaded, log an error and just use the last one received from that site.
::
:: = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
::
:_GET_HOST_LIST _HC_LIST_NAME  _HC_LIST_URL [_HC_FILE_NAME_INSIDE_ZIP]
::
    SET _HC_LIST_NAME=%1
    SET _HC_LIST_URL=%2
::
    ECHO #   %_HC_LIST_URL%>>Banner.txt
::
    IF '%3'=='' (SET _HC_EXT=txt) ELSE ((SET _HC_EXT=zip) & (SET _HC_FILE_NAME_INSIDE_ZIP=%3))
::
    wget --no-check-certificate -O %_HC_LIST_NAME%.%_HC_EXT% %_HC_LIST_URL%
    IF ERRORLEVEL 1 GOTO ERROR_GET_HOST_LIST
    IF "%_HC_EXT%" == "txt" GOTO CONTINUE_GET_HOST_LIST
::
    unzip %_HC_LIST_NAME%.%_HC_EXT%
    IF ERRORLEVEL 1 GOTO ERROR_GET_HOST_LIST
::
    COPY %_HC_FILE_NAME_INSIDE_ZIP% %_HC_LIST_NAME%.txt
::
:CONTINUE_GET_HOST_LIST
::
    ECHO.>>%_HC_LIST_NAME%.txt
::
::  Isolate just IPv4 server names.  Strip off any addresses from the beginning and port numbers or bracketed comments from the end.
::
    sed -r -e "s/#.*$//g" -e "s/[ \t]+/ /g" -e "s/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ //g" -e "s/:.*$//g" -e "s/\[.*$//g" -e "s/[ ]+$//g" -e "s/\n\n+/\n/g" -e "/^local$/d" -e "/^localhost$/d" -e "/^broadcasthost$/d" -e "/^localhost\.localdomain$/d" -e "/^[ \t]*$/d" %_HC_LIST_NAME%.txt >%_HC_LIST_NAME%1.tmp
    IF ERRORLEVEL 1 GOTO ERROR_GET_HOST_LIST
::
::  Add www. in front of entries that are just domain names (one dot) for completeness
::
    grep -E "^[^.]+\.[^.]+$" %_HC_LIST_NAME%1.tmp | sed -e "s/^/www./g" >%_HC_LIST_NAME%2.tmp
::
    COPY /Y /B %_HC_LIST_NAME%1.tmp+%_HC_LIST_NAME%2.tmp %_HC_LIST_NAME%.tmp >nul
    gsort -f -u %_HC_LIST_NAME%.tmp >%_HC_LIST_NAME%.list
::
    IF EXIST "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" COPY "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.last" >nul
    COPY %_HC_LIST_NAME%.list "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" >nul
::
    IF NOT EXIST "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.last" GOTO DONE_GET_HOST_LIST
    ECHO %_HC_LIST_NAME%.list vs. %_HC_LIST_NAME%.last
    ECHO -----------------------------------------------------------------------------------------------------------------------------
    diff -y --suppress-common-lines --width=125 "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.last"
    ECHO -----------------------------------------------------------------------------------------------------------------------------
    ECHO.
    GOTO DONE_GET_HOST_LIST
::
:ERROR_GET_HOST_LIST
::
    ECHO.
    ECHO ERROR:  %_HC_LIST_NAME% hosts list could not be retrieved; using existing %_HC_LIST_NAME%.list
    IF EXIST "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" COPY "%PROGRAMDATA%\HostsCompiler\%_HC_LIST_NAME%.list" .
::
:DONE_GET_HOST_LIST
::
    EXIT /B
::
::==============================================================================
::
:END_OF_PROCEDURE
::
::==============================================================================
::
::  End of file.
::
::==============================================================================
